@@ -1,8 +1,8 @@
 #!/usr/bin/python3
 
 #
-#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
-#   All rights reserved. Copyright (c) 2016-2020 VECTIONEER.
+#   Developer: Alexey Zakharov (alexey.zakharov@vectioneer.com)
+#   All rights reserved. Copyright (c) 2016-2025 VECTIONEER.
 #
 import base64
 import hashlib
@@ -11,6 +11,7 @@ import tempfile
 
 from motorcortex.reply import Reply
 from motorcortex.setup_logger import logger
+from motorcortex.state_callback_handler import StateCallbackHandler
 
 import os
 import queue
@@ -56,9 +57,9 @@ class Request(object):
         self.__connected_lock = None
         self.__protobuf_types = protobuf_types
         self.__parameter_tree = parameter_tree
-        self.__pool = ThreadPoolExecutor(max_workers=1)
-        self.__clb = ThreadPoolExecutor(max_workers=1)
         self.__connection_state = ConnectionState.DISCONNECTED
+        self.__pool = ThreadPoolExecutor(max_workers=1)
+        self.__callback_handler = StateCallbackHandler()
 
     def url(self):
         return self.__url
@@ -66,6 +67,10 @@ class Request(object):
     def connect(self, url, **kwargs):
         self.__connection_state = ConnectionState.CONNECTING
         conn_timeout_ms, recv_timeout_ms, certificate, state_update = self.parse(**kwargs)
+
+        if state_update:
+            self.__callback_handler.start(state_update)
+
         self.__url = url
         tls_config = None
         if certificate:
@@ -77,8 +82,7 @@ class Request(object):
         def pre_connect_cb(pipe):
             self.__connected_lock.put(True)
             self.__connection_state = ConnectionState.CONNECTION_OK
-            if state_update:
-                self.__clb.submit(state_update, self, self.connectionState())
+            self.__callback_handler.notify(self, self.connectionState())
 
         def post_remove_cb(pipe):
             if self.__connection_state == ConnectionState.DISCONNECTING:
@@ -88,8 +92,7 @@ class Request(object):
             elif self.__connection_state == ConnectionState.CONNECTION_OK:
                 self.__connection_state = ConnectionState.CONNECTION_LOST
             self.__connected_lock.put(False)
-            if state_update:
-                self.__clb.submit(state_update, self, self.connectionState())
+            self.__callback_handler.notify(self, self.connectionState())
 
         self.__socket.add_post_pipe_connect_cb(pre_connect_cb)
         self.__socket.add_post_pipe_remove_cb(post_remove_cb)
@@ -103,6 +106,7 @@ class Request(object):
         if self.__connected_lock:
             self.__connected_lock.put(False)
         self.__socket.close()
+        self.__callback_handler.stop()
 
     def send(self, encoded_msg, do_not_decode_reply=False):
         if self.__socket is not None:
@@ -119,7 +123,7 @@ class Request(object):
 
             Results:
                 Reply(StatusMsg): A Promise, which resolves if login is successful and fails otherwise.
-                Returned message has a status code, which indicates a status of the login.
+                The returned message has a status code, which indicates the status of the login.
 
             Examples:
                 >>> login_reply = req.login('operator', 'iddqd')
@@ -161,7 +165,7 @@ class Request(object):
         """Request a parameter tree from the server.
 
             Returns:
-                Reply(ParameterTreeMsg): A Promise, which resolves when parameter tree is received or fails
+                Reply(ParameterTreeMsg): A Promise, which resolves when a parameter tree is received or fails
                 otherwise. ParameterTreeMsg message has a status field to check the status of the operation.
 
             Examples:
@@ -194,14 +198,14 @@ class Request(object):
         return self.send(self.__protobuf_types.encode(param_save_msg))
 
     def setParameter(self, path, value, type_name=None, offset=0, length=0):
-        """Set new value to a parameter with given path
+        """Set a new value to a parameter with a given path
 
             Args:
                 path(str): parameter path in the tree
                 value(any): new parameter value
-                type_name(str): type of the value (by default resolved automatically)
-                offset(int): offset of the elements to update in the destination array, (by default is 0)
-                length(int): number of the elements to update in the destination array, (by default takes length of the
+                type_name(str): type of the value (by default, resolved automatically)
+                offset(int): offset of the elements to update in the destination array, (by default, is 0)
+                length(int): number of the elements to update in the destination array, (by default, takes length of the
                 value argument)
 
             Returns:
@@ -262,7 +266,7 @@ class Request(object):
                 path(str): parameter path in the tree.
 
             Returns:
-                 Reply(ParameterMsg): Returns a Promise, which resolves when parameter
+                 Reply(ParameterMsg): Returns a Promise, which resolves when a parameter
                  message is successfully obtained, fails otherwise.
 
             Examples:
@@ -276,10 +280,10 @@ class Request(object):
         """Get description and values of requested parameters.
 
             Args:
-                path_list(list(str)): list of parameter paths in the tree.
+                path_list(list(str)): a list of parameter paths in the tree.
 
             Returns:
-                Reply(ParameterListMsg): A Promise, which resolves when list of the parameter values is received, fails
+                Reply(ParameterListMsg): A Promise, which resolves when a list of the parameter values is received, fails
                 otherwise.
 
             Examples:
@@ -299,17 +303,17 @@ class Request(object):
 
     def overwriteParameter(self, path, value, force_activate=False, type_name=None):
         """Overwrites actual value of the parameter and depending on the flag forces this value to stay active.
-           This method of setting values is useful during debug and installation process, it is not recommended to use
+           This method of setting values is useful during the debug and installation process, it is not recommended to use
            this method during normal operation.
 
             Args:
                 path(str): parameter path in the tree
                 value(any): new parameter value
-                force_activate(bool): forces new value to stay active. (by default is set to 'False')
-                type_name(str): type of the value (by default resolved automatically)
+                force_activate(bool): forces the new value to stay active. (by default, is set to 'False')
+                type_name(str): type of the value (by default, resolved automatically)
 
             Returns:
-                  Reply(StatusMsg): A Promise, which resolves when parameter value is updated or fails otherwise.
+                  Reply(StatusMsg): A Promise, which resolves when a parameter value is updated or fails otherwise.
 
             Examples:
                   >>> reply = req.overwriteParameter("root/Control/dummyBool", False, True)
@@ -322,13 +326,13 @@ class Request(object):
                                                                                         self.__parameter_tree)))
 
     def releaseParameter(self, path):
-        """Deactivate overwrite operation of the parameter.
+        """Deactivate the overwrite operation of the parameter.
 
             Args:
                 path(str): parameter path in the tree
 
             Returns:
-                  Reply(StatusMsg): A Promise, which resolves when parameter value is released or fails otherwise.
+                  Reply(StatusMsg): A Promise, which resolves when a parameter value is released or fails otherwise.
 
             Examples:
                   >>> reply = req.releaseParameter("root/Control/dummyBool")
@@ -348,7 +352,7 @@ class Request(object):
                 frq_divider(int): frequency divider is a downscaling factor for the group publish rate
 
             Returns:
-                Reply(GroupStatusMsg): A Promise, which resolves when subscription is complete,
+                Reply(GroupStatusMsg): A Promise, which resolves when the subscription is complete,
                 fails otherwise.
         """
 
