@@ -1,19 +1,21 @@
 #!/usr/bin/python3
 
 #
-#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
-#   All rights reserved. Copyright (c) 2016 VECTIONEER.
+#   Developer: Alexey Zakharov (alexey.zakharov@vectioneer.com)
+#   All rights reserved. Copyright (c) 2016-2025 VECTIONEER.
 #
 
 from motorcortex.request import Request, Reply, ConnectionState
 from motorcortex.subscription import Subscription
+from motorcortex.state_callback_handler import StateCallbackHandler
+from motorcortex.setup_logger import logger
 from pynng import Sub0, TLSConfig
 from concurrent.futures import ThreadPoolExecutor
 from queue import Queue
 
 
 class Subscribe:
-    """Subscribe class is used to receive continuous parameter updates from motorcortex server.
+    """Subscribe class is used to receive continuous parameter updates from the motorcortex server.
 
         Subscribe class simplifies creating and removing subscription groups.
 
@@ -31,22 +33,26 @@ class Subscribe:
         self.__protobuf_types = protobuf_types
         self.__subscriptions = dict()
         self.__pool = ThreadPoolExecutor()
-        self.__clb = ThreadPoolExecutor(max_workers=1)
+        self.__callback_handler = StateCallbackHandler()
         self.__connection_state = ConnectionState.DISCONNECTED
 
     def connect(self, url, **kwargs):
-        """Open a subscribe connection.
+        """Open a subscription connection.
 
             Args:
                 url(str): motorcortex server URL
 
             Returns:
-                bool: True - if connected, False othesudo apt-get install python-devrwise
+                bool: True - if connected, False otherwise
         """
 
         self.__connection_state = ConnectionState.CONNECTING
         conn_timeout_ms, recv_timeout_ms, certificate, state_update = Request.parse(**kwargs)
-        if recv_timeout_ms == 0:
+
+        if state_update:
+            self.__callback_handler.start(state_update)
+
+        if not recv_timeout_ms:
             recv_timeout_ms = 500
 
         self.__url = url
@@ -61,8 +67,7 @@ class Subscribe:
         def pre_connect_cb(pipe):
             self.__connected_lock.put(True)
             self.__connection_state = ConnectionState.CONNECTION_OK
-            if state_update:
-                self.__clb.submit(state_update, self.__req, self, self.connectionState())
+            self.__callback_handler.notify(self.__req, self, self.connectionState())
 
         def post_remove_cb(pipe):
             if self.__connection_state == ConnectionState.DISCONNECTING:
@@ -72,8 +77,7 @@ class Subscribe:
             elif self.__connection_state == ConnectionState.CONNECTION_OK:
                 self.__connection_state = ConnectionState.CONNECTION_LOST
             self.__connected_lock.put(False)
-            if state_update:
-                self.__clb.submit(state_update, self.__req, self, self.connectionState())
+            self.__callback_handler.notify(self.__req, self, self.connectionState())
 
         self.__socket.add_pre_pipe_connect_cb(pre_connect_cb)
         self.__socket.add_post_pipe_remove_cb(post_remove_cb)
@@ -90,6 +94,7 @@ class Subscribe:
         if self.__connected_lock:
             self.__connected_lock.put(False)
         self.__socket.close()
+        self.__callback_handler.stop()
         self.__pool.shutdown(wait=True)
 
     def run(self, socket):
@@ -107,9 +112,9 @@ class Subscribe:
                     elif protocol_version == 0:
                         sub._updateProtocol0(buffer[4:], length - 4)
                     else:
-                        print('Unknown protocol type: %d', protocol_version)
+                        logger.error(f'Unknown protocol version: {protocol_version}')
 
-        print('Subscribe connection closed')
+        logger.debug('Subscribe connection closed')
 
     def subscribe(self, param_list, group_alias, frq_divider=1):
         """Create a subscription group for a list of the parameters.
@@ -121,11 +126,11 @@ class Subscribe:
 
             Returns:
                   Subscription: A subscription handle, which acts as a JavaScript Promise,
-                  it is resolved when subscription is ready or failed. After the subscription
-                  is ready the handle is used to retrieve latest data.
+                  it is resolved when the subscription is ready or failed. After the subscription
+                  is ready, the handle is used to retrieve the latest data.
         """
 
-        subscription = Subscription(group_alias, self.__protobuf_types, frq_divider)
+        subscription = Subscription(group_alias, self.__protobuf_types, frq_divider, self.__pool)
         reply = self.__req.createGroup(param_list, group_alias, frq_divider)
         reply.then(self.__complete, subscription, self.__socket).catch(
             subscription._failed)
